/*
 * import.h: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#ifndef _IMPORT_H
#define _IMPORT_H

#include <vdr/channels.h>
#include <sqlite3.h>

#include "xmltv2vdr.h"
#include "maps.h"

enum
{
    IMPORT_ALL=0,
    IMPORT_DESCRIPTION,
    IMPORT_SHORTTEXT
};

class cImport
{
private:
    struct split
    {
        char *pointers[256];
        int count;
    };
    enum
    {
        IMPORT_NOERROR=0,
        IMPORT_NOSCHEDULE,
        IMPORT_NOCHANNEL,
        IMPORT_XMLTVERR,
        IMPORT_NOMAPPING,
        IMPORT_NOCHANNELID,
        IMPORT_EMPTYSCHEDULE
    };
    cEPGMappings *maps;
    cEPGSource *source;
    cTEXTMappings *texts;
    cCharSetConv *conv;
    const char *epgfile;
    char *RemoveLastCharFromDescription(char *description);
    char *Add2Description(char *description, const char *Value);

    char *Add2Description(char *description, const char *Name, const char *Value);
    char *Add2Description(char *description, const char *Name, int Value);
    cTEXTMapping *TEXTMapping(const char *Name);
    cEPGMapping *EPGMapping(const char *ChannelName);
    struct split split(char *in, char delim);
    cEvent *GetEventBefore(cSchedule* schedule, time_t start);
    cEvent *SearchVDREvent(cSchedule* schedule, cXMLTVEvent *event);
    bool FetchXMLTVEvent(sqlite3_stmt *stmt, cXMLTVEvent *xevent);
    char *RemoveNonASCII(const char *src);
    cXMLTVEvent *PrepareAndReturn(sqlite3 *db, char *sql, sqlite3_stmt *stmt);
public:
    cImport(cEPGSource *Source, cEPGMappings *Maps, cTEXTMappings *Texts);
    ~cImport();
    int Process(cEPGExecutor &myExecutor);
    bool PutEvent(cEPGSource *source, sqlite3 *db, cSchedule* schedule, cEvent *event,
                  cXMLTVEvent *xevent, cEPGMapping *map, int Option=IMPORT_ALL);
    cEPGMapping *EPGMapping(tChannelID ChannelID);
    cXMLTVEvent *SearchXMLTVEvent(const char *EPGFile, cEvent *event);
    void UpdateXMLTVEvent(const char *EPGFile, sqlite3 *db, const char *Source, tEventID EventID,
                          tEventID EITEventID, const char *EITDescription=NULL);
};

#endif
