/*
 * parse.h: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#ifndef _PARSE_H
#define _PARSE_H

#include <vdr/epg.h>
#include <libxml/parser.h>
#include <time.h>

#include "maps.h"
#include "event.h"

class cEPGExecutor;
class cEPGSource;

class cParse
{
    enum
    {
        PARSE_NOERROR=0,
        PARSE_XMLTVERR,
        PARSE_NOMAPPING,
        PARSE_NOCHANNELID
    };

private:
    iconv_t conv;
    char *epdir;
    cEPGSource *source;
    cEPGMappings *maps;
    cXMLTVEvent xevent;
    bool FetchSeasonEpisode(const char *epdir, const char *title, const char *shorttext, int &season, int &episode);
    cEPGMapping *EPGMapping(const char *ChannelName);
    time_t ConvertXMLTVTime2UnixTime(char *xmltvtime);
    void RemoveNonAlphaNumeric(char *string);
    bool FetchEvent(xmlNodePtr node);
public:
    cParse(cEPGSource *Source, cEPGMappings *Maps);
    ~cParse();
    int Process(cEPGExecutor &myExecutor, char *buffer, int bufsize);
    static void InitLibXML();
    static void CleanupLibXML();
};

#endif
